<?php

namespace Drupal\par;

use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ParService.
 */
class ParService {

  /**
   * Constants for access statuses marking.
   */
  const PAR_ACCESS_ALLOWED = 0;
  const PAR_ACCESS_DENIED = 1;
  const PAR_NOT_FOUND = 2;

  /**
   * The database connection.
   *
   * @var Connection
   */
  protected $database;

  /**
   * The current user.
   *
   * @var AccountInterface
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Create dependency injection for class.
   *
   * @param ContainerInterface $container
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('current_user'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Constructs a new DatabaseLockBackend.
   *
   * @param Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database, AccountInterface $currentUser, EntityTypeManagerInterface $entityTypeManager) {
    $this->database = $database;
    $this->currentUser = $currentUser;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Writes new record to the par table.
   *
   * @param $entity_type
   * @param $entity_id
   * @param int $action
   * @param array $uids
   * @param array $roles
   */
  public function addRecords($entity_type, $entity_id, $action = 0, $uids = [], $roles = []) {
    $fields = [
      'entity_type' => $entity_type,
      'entity_id' => $entity_id,
      'action' => $action,
    ];
    foreach ($uids as $uid) {
      if ($uid) {
        $this->database->insert('par')
          ->fields($fields + [
              'uid' => (int) $uid,
            ])
          ->execute();
      }
    }
    foreach ($roles as $role) {
      if ($role) {
        $this->database->insert('par')
          ->fields($fields + [
              'role' => $role,
            ])
          ->execute();
      }
    }
  }

  /**
   * Update existing records for certain content.
   *
   * @param $entity_type
   * @param $entity_id
   * @param int $action
   * @param array $uids
   * @param array $roles
   */
  public function updateRecords($entity_type, $entity_id, $action = 0, $uids = [], $roles = []) {
    $records = $this->database->select('par', 'p')
      ->fields('p')
      ->condition('entity_type', $entity_type)
      ->condition('entity_id', $entity_id)
      ->execute()
      ->fetchAll();

    if ($records) {
      $this->database->delete('par')
        ->condition('entity_type', $entity_type)
        ->condition('entity_id', $entity_id)
        ->execute();
    }

    $this->addRecords($entity_type, $entity_id, $action, $uids, $roles);
  }

  /**
   * Get Records by Entity ID.
   *
   * @param $entity_id
   * @return mixed
   */
  public function getRecordsByEntityId($entity_id) {
    return $this->database->select('par', 'p')
      ->fields('p')
      ->condition('entity_id', $entity_id)
      ->execute()
      ->fetchAll();
  }

  /**
   * get records by User ID.
   *
   * @param null $uid
   * @return mixed
   */
  public function getRecordsByUserId($uid = NULL) {
    if (!$uid) {
      $uid = $this->currentUser->id();
    }

    return $this->database->select('par', 'p')
      ->fields('p')
      ->condition('uid', $uid)
      ->execute()
      ->fetchAll();
  }

  /**
   * Get Records using any params.
   *
   * @param string $entity_type
   * @param int $entity_id
   * @param null $uid
   * @param null $role
   * @param null $action
   * @return mixed
   */
  public function getRecords($entity_type = '', $entity_id = 0, $uid = 0, $role = '', $action = '') {
    $query = $this->database->select('par', 'p')
      ->fields('p');

    if ($entity_id) {
      $query->condition('entity_id', $entity_id);
    }
    if ($entity_type) {
      $query->condition('entity_type', $entity_type);
    }
    if ($uid) {
      $query->condition('uid', $uid);
    }
    if ($role) {
      $query->condition('role', $role);
    }
    if ($action) {
      $query->condition('action', $action);
    }

    return $query->execute()->fetchAll();
  }

  /**
   * Returns access content status for a particular entity item.
   *
   * @param $entity_type
   * @param $entity_id
   * @param null $uid
   * @return int
   */
  public function getContentAccessStatus($entity_type, $entity_id, $uid = 0) {
    $user = $uid ? $this->entityTypeManager->getStorage('user')->load($uid) : $this->currentUser;
    $uid = $user->id();
    $role = ($uid == 0) ? 'anonymous' : '';
    $is_admin = in_array('administrator', $user->getRoles());
    $has_view_permission = $user->hasPermission('view restricted pages');
    if ($is_admin || $has_view_permission) {
      return self::PAR_ACCESS_ALLOWED;
    }

    $records = $this->getRecords($entity_type, $entity_id, $uid, $role);
    if (!$records) {
      return self::PAR_ACCESS_ALLOWED;
    }
    $record = reset($records);

    return $record->action;
  }

  /**
   * Returns label of the access ID code.
   *
   * @param $id
   * @return mixed
   */
  public function getAccessLabelById($id) {
    $ids = [
      self::PAR_ACCESS_ALLOWED => t('Access allowed'),
      self::PAR_ACCESS_DENIED => t('Access denied'),
      self::PAR_NOT_FOUND => t('Page not found'),
    ];

    return $ids[$id];
  }

}
